==> libecsb-0.0.19.tar.gz: libecsb-0.0.19/ecsb/Misc.cpp <==
// random number generation functions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "Misc.h"

#include <ctime>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <cassert>
#include <sstream>
#include <list>

using namespace ECSB;

// generate human readable timestamp string
const char *ECSB::generateTimestamp(void)
{
	struct timeval rawtime;
	struct tm *timeinfo;
	static char buffer[255];
	char usecs[255];

	gettimeofday(&rawtime, NULL);  // record end time
	timeinfo = localtime(&(rawtime.tv_sec));
	strftime(buffer, 255, "%d/%m/%y-%H:%M:%S %s", timeinfo);
	snprintf(usecs, 255, "%f", rawtime.tv_usec / 10.0E6);
	strncat(buffer, strchr(usecs, '.'), 255);
	return buffer;
}

// convert time in seconds to human readable string
const char *ECSB::convert_time(const unsigned long time)
{
	static char output[255];

	if(time < 60) {
		snprintf(output, 255, "%ld second%s", time, (time == 1) ? "" : "s");
		return output;
	}
	if(time < 3600) {
		snprintf(output, 255, "%ld minute%s %ld second%s", time / 60, \
				((time / 60) == 1) ? "" : "s", time % 60, \
				((time % 60) == 1) ? "" : "s");
		return output;
	}
	snprintf(output, 255, "%ld hour%s %ld minute%s %ld second%s", time / 3600, \
			((time / 3600) == 1) ? "" : "s", (time % 3600) / 60, \
			(((time % 3600) / 60) == 1) ? "" : "s", (time % 3600) % 60, \
			(((time % 3600) % 60) == 1) ? "" : "s");
	return output;
}

// convert time in seconds and fractions of seconds to human readable string
const char *ECSB::convert_time(const double time)
{
	static char output[255];

	if(time == floor(time))
		return convert_time((unsigned long) time);
	if(time < 60) {
		snprintf(output, 255, "%lf second%s", time, (time == 1.0) ? "" : "s");
		return output;
	}
	unsigned long ultime = (unsigned long) time;
	if(ultime < 3600) {
		snprintf(output, 255, "%ld minute%s %lf second%s", ultime / 60, \
				((ultime / 60) == 1) ? "" : "s", \
				fmod(time, 60.0), \
				(fmod(time, 60.0) == 1) ? "" : "s");
		return output;
	}
	snprintf(output, 255, "%ld hour%s %ld minute%s %lf second%s", \
			ultime / 3600, ((ultime / 3600) == 1) ? "" : "s", \
			(ultime % 3600) / 60, (((ultime % 3600) / 60) == 1) ? "" : "s", \
			fmod(fmod(time, 3600.0), 60.0), \
			(fmod(fmod(time, 3600.0), 60.0) == 1) ? "" : "s");
	return output;
}

// compute difference in seconds between two timevals
double ECSB::timeval_diff(struct timeval &from, struct timeval &to)
{
	long double time1, time2;

	time1 = from.tv_sec + from.tv_usec / 10E6;
	time2 = to.tv_sec + to.tv_usec / 10E6;
	return ((double) (time2 - time1));
}

// compute difference in seconds between two timestamps
double ECSB::timestamp_diff(const char *from, const char *to)
{
 	return (atof(strchr(to, ' ') + 1) - atof(strchr(from, ' ') + 1));
}

// return date from timestamp
const char *ECSB::timestamp_date(char *time)
{
	static char output[255], *ptr1;

	ptr1 = strchr(time, '-');
	strncpy(output, time, ptr1 - time);
	output[ptr1 - time] = '\0';
	return output;
}

// return time from timestamp
const char *ECSB::timestamp_time(char *time)
{
	static char output[255], *ptr1, *ptr2;

	ptr1 = strchr(time, '-') + 1;
	ptr2 = strchr(time, ' ');
	strncpy(output, ptr1, ptr2 - ptr1);
	output[ptr2 - ptr1] = '\0';
	return output;
}

// stochastic comparison event (greater than threshold)
double stochastic_comparison_GT(double E, double X, double T1, double T2)
{
	int *event;
	memcpy(&event, &E, sizeof(int *));

	if(X < T1) {
		*event = 1;
		return false;
	}
	if(X > T2 && *event == 1) {
		*event = 0;
		return true;
	}
	return false;
}

// stochastic comparison event (greater than or equal to threshold)
double stochastic_comparison_GE(double E, double X, double T1, double T2)
{
	int *event;
	memcpy(&event, &E, sizeof(int *));

	if(X < T1) {
		*event = 1;
		return false;
	}
	if(X >= T2 && *event == 1) {
		*event = 0;
		return true;
	}
	return false;
}

// stochastic comparison event (less than threshold)
double stochastic_comparison_LT(double E, double X, double T1, double T2)
{
	int *event;
	memcpy(&event, &E, sizeof(int *));

	if(X > T1) {
		*event = 1;
		return false;
	}
	if(X < T2 && *event == 1) {
		*event = 0;
		return true;
	}
	return false;
}

// stochastic comparison event (less than or equal to threshold)
double stochastic_comparison_LE(double E, double X, double T1, double T2)
{
	int *event;
	memcpy(&event, &E, sizeof(int *));

	if(X > T1) {
		*event = 1;
		return false;
	}
	if(X <= T2 && *event == 1) {
		*event = 0;
		return true;
	}
	return false;
}

// initialise stochastic comparison events
void initialise_stochastic_comparisons(mu::Parser &parser)
{
	char *ptr1, *ptr2, *ptr3, *expr, *expr2, *endptr;
	char X[1024], T[1024], newsc[1024], varname[1024];
	double Tf, *E;
	int *event;
	unsigned long len2, len3, num_events;
	bool gt, eq;

	// check pointer storage sufficient
	if(sizeof(int *) > sizeof(double)) {
		std::cerr << "error: unable to implement stochastic events\n";
		exit(EXIT_FAILURE);
	}

	// create temporary buffer
	if(!(expr = (char *) malloc((strlen(parser.GetExpr().c_str()) + 1) \
			* sizeof(char)))) {
		std::cerr << "error: unable to initialise stochastic events\n";
		exit(EXIT_FAILURE);
	}
	strcpy(expr, parser.GetExpr().c_str());

	// create temporary buffer
	if(!(expr2 = (char *) malloc(1024 * sizeof(char)))) {
		std::cerr << "error: unable to initialise stochastic events\n";
		exit(EXIT_FAILURE);
	}

	// replace stochastic comparison functions
	ptr1 = expr;
	ptr2 = expr2;
	len2 = 0;
	num_events = 0;
	while((ptr3 = strstr(ptr1, "SC"))) {
		if(!(ptr3[2] == 'G' || ptr3[2] == 'L') || \
				!(ptr3[3] != 'T' || ptr3[3] != 'E') || ptr3[4] != '(') {
			// continue parsing expression
			len3 = ptr3 - ptr1;
			if((len2 += len3) >= 1024) {
				std::cerr << "error: unable to parse stochastic event in " \
						<< expr << "\n";
				exit(EXIT_FAILURE);
			}
			strncpy(ptr2, ptr1, len3);
			ptr2 += len3;
			ptr1 = ptr3 + 2;
			continue;
		}
		if(ptr3[2] == 'G')
			gt = true;
		else
			gt = false;
		if(ptr3[2] == 'E')
			eq = true;
		else
			eq = false;
		// copy preceeding part of expression
		len3 = ptr3 - ptr1;
		if((len2 += len3) >= 1024) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		strncpy(ptr2, ptr1, len3);
		ptr2 += len3;
		ptr1 = ptr3 + 5;
		// extract variable
		if(!(ptr3 = strchr(ptr1, ',')) || (ptr3 - ptr1) >= 1024) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		strncpy(X, ptr1, ptr3 - ptr1);
		X[ptr3 - ptr1] = '\0';
		ptr1 = ptr3 + 1;
		// extract threshold
		if(!(ptr3 = strchr(ptr1, ')')) || (ptr3 - ptr1) >= 1024) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		strncpy(T, ptr1, ptr3 - ptr1);
		T[ptr3 - ptr1] = '\0';
		ptr1 = ptr3 + 1;
		// convert threshold
		Tf = strtod(T, &endptr);
		if(errno != 0 || endptr == T || *endptr != '\0' || Tf <= 0.0) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		// create event variable
		if(!(E = (double *) malloc(sizeof(double)))) {
			std::cerr << "error: unable to create stochastic events\n";
			exit(EXIT_FAILURE);
		}
		if(!(event = (int *) malloc(sizeof(int)))) {
			std::cerr << "error: unable to create stochastic events\n";
			exit(EXIT_FAILURE);
		}
		*event = 0;
		snprintf(varname, 1024, "event%ld", num_events);
		memcpy(E, &event, sizeof(int *));
		// create new comparison function
		if(gt) {
			if(eq)
				len3 = snprintf(newsc, 1024, "SCGE(%s,%s,%f,%s)", \
						varname, X, Tf - sqrt(Tf), T);
			else
				len3 = snprintf(newsc, 1024, "SCGT(%s,%s,%f,%s)", \
						varname, X, Tf - sqrt(Tf), T);
		} else {
			if(eq)
				len3 = snprintf(newsc, 1024, "SCLE(%s,%s,%f,%s)", \
						varname, X, Tf + sqrt(Tf), T);
			else
				len3 = snprintf(newsc, 1024, "SCLT(%s,%s,%f,%s)", \
						varname, X, Tf + sqrt(Tf), T);
		}
		if(len3 >= 1024) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		if((len3 >= 1024) || ((len2 += len3)) >= 1024) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		// set variable name
		parser.DefineVar(varname, E);
		num_events++;
		// add comparison function to expression
		strncpy(ptr2, newsc, len3 + 1);
		ptr2 += len3;
	}

	// copy remainder of expression
	if(strlen(ptr1) >= (1024 - len2)) {
		std::cerr << "error: unable to parse stochastic event in " \
				<< expr << "\n";
		exit(EXIT_FAILURE);
	}
	strncpy(ptr2, ptr1, 1024 - len2);

	// replace expression
	parser.SetExpr(expr2);
    parser.DefineFun("SCGT", stochastic_comparison_GT);
    parser.DefineFun("SCGE", stochastic_comparison_GE);
    parser.DefineFun("SCLT", stochastic_comparison_LT);
    parser.DefineFun("SCLE", stochastic_comparison_LE);

	// free memory
	free(expr);
	free(expr2);
}

// reassign stochastic comparison event variables
void update_stochastic_comparisons_variables(mu::Parser &parser)
{
	int *event, *event_orig;
	double *E;
	const char *varname;
	mu::varmap_type variables;
	mu::varmap_type::const_iterator iter;

	// find all event variables
	variables = parser.GetUsedVar();
	for(iter = variables.begin(); iter != variables.end(); ++iter) {
		varname = iter->first.c_str();
		if(strstr(varname, "event") == varname) {
			// create new event variable
			if(!(E = (double *) malloc(sizeof(double)))) {
				std::cerr << "error: unable to create stochastic events\n";
				exit(EXIT_FAILURE);
			}
			if(!(event = (int *) malloc(sizeof(int)))) {
				std::cerr << "error: unable to create stochastic events\n";
				exit(EXIT_FAILURE);
			}
			memcpy(&event_orig, &(iter->second), sizeof(int *));
			*event = *event_orig;
			memcpy(E, &event, sizeof(int *));
			// reassign event variable
			parser.RemoveVar(varname);
			parser.DefineVar(varname, E);
		}
	}
}

// free stochastic comparison event variables
void free_stochastic_comparisons_variables(mu::Parser &parser)
{
	int *event;
	double *E;
	const char *varname;
	mu::varmap_type variables;
	mu::varmap_type::const_iterator iter;

	// find all event variables
	variables = parser.GetUsedVar();
	for(iter = variables.begin(); iter != variables.end(); ++iter) {
		varname = iter->first.c_str();
		if(strstr(varname, "event") == varname) {
			// create new event variable
			E = iter->second;
			memcpy(&event, E, sizeof(int *));
			free(E);
			free(event);
		}
	}
}

//Auxiliar substitution
std::map<std::string,std::string> ECSB::createSubstitution(std::vector<std::string> v1,
							 std::vector<std::string> v2)
{
  std::map<std::string,std::string> substitutionMap;

  if(v1.size() != v2.size())
    {
      std::cout << "Substitution vectors of different sizes for" << std::endl;
      std::cout << "v1 = ";
      for(std::vector<std::string>::iterator v = v1.begin(); v != v1.end(); ++v)
	std::cout << " " << *v;
      std::cout << std::endl <<  "v2 = ";
      for(std::vector<std::string>::iterator v = v2.begin(); v != v2.end(); ++v)
	std::cout << " " << *v;
      std::cout << std::endl;
      exit(1);
    }
  else
    {
      for(unsigned long i = 0; i != v1.size(); ++i)
	substitutionMap.insert(std::pair<std::string,std::string>(v1[i],v2[i]));
    }

  return substitutionMap;
}

//Auxiliar tokenizer
std::vector<std::string> ECSB::convertToVector(std::string str, std::string delimiters)
{
  std::vector<std::string> tokens;

  std::string::size_type pos_ini = str.find_first_not_of(delimiters,0);
  std::string::size_type pos_end = str.find_first_of(delimiters,pos_ini);

  while(pos_ini != std::string::npos || pos_end != std::string::npos)
    {
      tokens.push_back(str.substr(pos_ini,pos_end-pos_ini));
      pos_ini = str.find_first_not_of(delimiters,pos_end);
      pos_end = str.find_first_of(delimiters,pos_ini);
    }

  return tokens;
}

//Auxiliar vectors equality
int ECSB::equalityVector(std::vector<std::string> v1, std::vector<std::string> v2)
{
  if(v1.empty() && v2.empty())
    return 1;
  else if (v1.empty() && not(v2.empty()))
    return 0;
  else if (not(v1.empty()) && v2.empty())
    return 0;
  else
    {
      std::list<std::string> l1(v1.begin(),v1.end());
      l1.sort();
      std::list<std::string> l2(v2.begin(),v2.end());
      l2.sort();
      if(l1 == l2)
	return 1;
      else
	return 0;
    }
}

bool ECSB::isNumeric(std::string numericString)
{
     for(int i = 0; i < numericString.length(); ++i)
    {
        if(!(isdigit(numericString[i]) | numericString[i] == '.'))
        {
            return false; //Failure!!
        }
    }
   return true; //Success!!
}

==> libecsb-0.0.24.tar.gz: libecsb-0.0.24/ecsb/Misc.cpp <==

#include <vector>
#include <sbml/Compartment.h>

// random number generation functions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "Misc.h"

#include <ctime>
#include <map>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <cassert>
#include <sstream>
#include <fstream>
#include <list>

#include <sbml/SBMLTypes.h>


using namespace ECSB;

// generate human readable timestamp string
const char *ECSB::generateTimestamp(void)
{
	struct timeval rawtime;
	struct tm *timeinfo;
	static char buffer[255];
	char usecs[255];

	gettimeofday(&rawtime, NULL);  // record end time
	timeinfo = localtime(&(rawtime.tv_sec));
	strftime(buffer, 255, "%d/%m/%y-%H:%M:%S %s", timeinfo);
	snprintf(usecs, 255, "%f", rawtime.tv_usec / 10.0E6);
	strncat(buffer, strchr(usecs, '.'), 255);
	return buffer;
}

// convert time in seconds to human readable string
const char *ECSB::convert_time(const unsigned long time)
{
	static char output[255];

	if(time < 60) {
		snprintf(output, 255, "%u second%s", (unsigned int) time, (time == 1) ? "" : "s");
		return output;
	}
	if(time < 3600) {
		snprintf(output, 255, "%u minute%s %u second%s", (unsigned int) time / 60, \
				((time / 60) == 1) ? "" : "s", (unsigned int) time % 60, \
				((time % 60) == 1) ? "" : "s");
		return output;
	}
	snprintf(output, 255, "%u hour%s %u minute%s %u second%s", (unsigned int) time / 3600, \
			((time / 3600) == 1) ? "" : "s", (unsigned int) (time % 3600) / 60, \
			(((time % 3600) / 60) == 1) ? "" : "s", (unsigned int) (time % 3600) % 60, \
			(((time % 3600) % 60) == 1) ? "" : "s");
	return output;
}

// convert time in seconds and fractions of seconds to human readable string
const char *ECSB::convert_time(const double time)
{
	static char output[255];

	if(time == floor(time))
		return convert_time((unsigned long) time);
	if(time < 60) {
		snprintf(output, 255, "%f second%s", time, (time == 1.0) ? "" : "s");
		return output;
	}
	unsigned long ultime = (unsigned long) time;
	if(ultime < 3600) {
		snprintf(output, 255, "%u minute%s %f second%s", (unsigned int) ultime / 60, \
				((ultime / 60) == 1) ? "" : "s", \
				fmod(time, 60.0), \
				(fmod(time, 60.0) == 1) ? "" : "s");
		return output;
	}
	snprintf(output, 255, "%u hour%s %u minute%s %f second%s", \
			(unsigned int) ultime / 3600, ((ultime / 3600) == 1) ? "" : "s", \
			(unsigned int) (ultime % 3600) / 60, (((ultime % 3600) / 60) == 1) ? "" : "s", \
			fmod(fmod(time, 3600.0), 60.0), \
			(fmod(fmod(time, 3600.0), 60.0) == 1) ? "" : "s");
	return output;
}

// compute difference in seconds between two timevals
double ECSB::timeval_diff(struct timeval &from, struct timeval &to)
{
	long double time1, time2;

	time1 = from.tv_sec + from.tv_usec / 10E6;
	time2 = to.tv_sec + to.tv_usec / 10E6;
	return ((double) (time2 - time1));
}

// compute difference in seconds between two timestamps
double ECSB::timestamp_diff(const char *from, const char *to)
{
 	return (atof(strchr(to, ' ') + 1) - atof(strchr(from, ' ') + 1));
}

// return date from timestamp
const char *ECSB::timestamp_date(char *time)
{
	static char output[255], *ptr1;

	ptr1 = strchr(time, '-');
	strncpy(output, time, ptr1 - time);
	output[ptr1 - time] = '\0';
	return output;
}

// return time from timestamp
const char *ECSB::timestamp_time(char *time)
{
	static char output[255], *ptr1, *ptr2;

	ptr1 = strchr(time, '-') + 1;
	ptr2 = strchr(time, ' ');
	strncpy(output, ptr1, ptr2 - ptr1);
	output[ptr2 - ptr1] = '\0';
	return output;
}

// stochastic comparison event (greater than threshold)
double stochastic_comparison_GT(double E, double X, double T1, double T2)
{
	int *event;
	memcpy(&event, &E, sizeof(int *));

	if(X < T1) {
		*event = 1;
		return false;
	}
	if(X > T2 && *event == 1) {
		*event = 0;
		return true;
	}
	return false;
}

// stochastic comparison event (greater than or equal to threshold)
double stochastic_comparison_GE(double E, double X, double T1, double T2)
{
	int *event;
	memcpy(&event, &E, sizeof(int *));

	if(X < T1) {
		*event = 1;
		return false;
	}
	if(X >= T2 && *event == 1) {
		*event = 0;
		return true;
	}
	return false;
}

// stochastic comparison event (less than threshold)
double stochastic_comparison_LT(double E, double X, double T1, double T2)
{
	int *event;
	memcpy(&event, &E, sizeof(int *));

	if(X > T1) {
		*event = 1;
		return false;
	}
	if(X < T2 && *event == 1) {
		*event = 0;
		return true;
	}
	return false;
}

// stochastic comparison event (less than or equal to threshold)
double stochastic_comparison_LE(double E, double X, double T1, double T2)
{
	int *event;
	memcpy(&event, &E, sizeof(int *));

	if(X > T1) {
		*event = 1;
		return false;
	}
	if(X <= T2 && *event == 1) {
		*event = 0;
		return true;
	}
	return false;
}

// initialise stochastic comparison events
void initialise_stochastic_comparisons(mu::Parser &parser)
{
	char *ptr1, *ptr2, *ptr3, *expr, *expr2, *endptr;
	char X[1024], T[1024], newsc[1024], varname[1024];
	double Tf, *E;
	int *event;
	unsigned long len2, len3, num_events;
	bool gt, eq;

	// check pointer storage sufficient
	if(sizeof(int *) > sizeof(double)) {
		std::cerr << "error: unable to implement stochastic events\n";
		exit(EXIT_FAILURE);
	}

	// create temporary buffer
	if(!(expr = (char *) malloc((strlen(parser.GetExpr().c_str()) + 1) \
			* sizeof(char)))) {
		std::cerr << "error: unable to initialise stochastic events\n";
		exit(EXIT_FAILURE);
	}
	strcpy(expr, parser.GetExpr().c_str());

	// create temporary buffer
	if(!(expr2 = (char *) malloc(1024 * sizeof(char)))) {
		std::cerr << "error: unable to initialise stochastic events\n";
		exit(EXIT_FAILURE);
	}

	// replace stochastic comparison functions
	ptr1 = expr;
	ptr2 = expr2;
	len2 = 0;
	num_events = 0;
	while((ptr3 = strstr(ptr1, "SC"))) {
		if(!(ptr3[2] == 'G' || ptr3[2] == 'L') || \
				!(ptr3[3] != 'T' || ptr3[3] != 'E') || ptr3[4] != '(') {
			// continue parsing expression
			len3 = ptr3 - ptr1;
			if((len2 += len3) >= 1024) {
				std::cerr << "error: unable to parse stochastic event in " \
						<< expr << "\n";
				exit(EXIT_FAILURE);
			}
			strncpy(ptr2, ptr1, len3);
			ptr2 += len3;
			ptr1 = ptr3 + 2;
			continue;
		}
		if(ptr3[2] == 'G')
			gt = true;
		else
			gt = false;
		if(ptr3[2] == 'E')
			eq = true;
		else
			eq = false;
		// copy preceeding part of expression
		len3 = ptr3 - ptr1;
		if((len2 += len3) >= 1024) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		strncpy(ptr2, ptr1, len3);
		ptr2 += len3;
		ptr1 = ptr3 + 5;
		// extract variable
		if(!(ptr3 = strchr(ptr1, ',')) || (ptr3 - ptr1) >= 1024) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		strncpy(X, ptr1, ptr3 - ptr1);
		X[ptr3 - ptr1] = '\0';
		ptr1 = ptr3 + 1;
		// extract threshold
		if(!(ptr3 = strchr(ptr1, ')')) || (ptr3 - ptr1) >= 1024) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		strncpy(T, ptr1, ptr3 - ptr1);
		T[ptr3 - ptr1] = '\0';
		ptr1 = ptr3 + 1;
		// convert threshold
		Tf = strtod(T, &endptr);
		if(errno != 0 || endptr == T || *endptr != '\0' || Tf <= 0.0) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		// create event variable
		if(!(E = (double *) malloc(sizeof(double)))) {
			std::cerr << "error: unable to create stochastic events\n";
			exit(EXIT_FAILURE);
		}
		if(!(event = (int *) malloc(sizeof(int)))) {
			std::cerr << "error: unable to create stochastic events\n";
			exit(EXIT_FAILURE);
		}
		*event = 0;
		snprintf(varname, 1024, "event%ld", num_events);
		memcpy(E, &event, sizeof(int *));
		// create new comparison function
		if(gt) {
			if(eq)
				len3 = snprintf(newsc, 1024, "SCGE(%s,%s,%f,%s)", \
						varname, X, Tf - sqrt(Tf), T);
			else
				len3 = snprintf(newsc, 1024, "SCGT(%s,%s,%f,%s)", \
						varname, X, Tf - sqrt(Tf), T);
		} else {
			if(eq)
				len3 = snprintf(newsc, 1024, "SCLE(%s,%s,%f,%s)", \
						varname, X, Tf + sqrt(Tf), T);
			else
				len3 = snprintf(newsc, 1024, "SCLT(%s,%s,%f,%s)", \
						varname, X, Tf + sqrt(Tf), T);
		}
		if(len3 >= 1024) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		if((len3 >= 1024) || ((len2 += len3)) >= 1024) {
			std::cerr << "error: unable to parse stochastic event in " \
					<< expr << "\n";
			exit(EXIT_FAILURE);
		}
		// set variable name
		parser.DefineVar(varname, E);
		num_events++;
		// add comparison function to expression
		strncpy(ptr2, newsc, len3 + 1);
		ptr2 += len3;
	}

	// copy remainder of expression
	if(strlen(ptr1) >= (1024 - len2)) {
		std::cerr << "error: unable to parse stochastic event in " \
				<< expr << "\n";
		exit(EXIT_FAILURE);
	}
	strncpy(ptr2, ptr1, 1024 - len2);

	// replace expression
	parser.SetExpr(expr2);
    parser.DefineFun("SCGT", stochastic_comparison_GT);
    parser.DefineFun("SCGE", stochastic_comparison_GE);
    parser.DefineFun("SCLT", stochastic_comparison_LT);
    parser.DefineFun("SCLE", stochastic_comparison_LE);

	// free memory
	free(expr);
	free(expr2);
}

// reassign stochastic comparison event variables
void update_stochastic_comparisons_variables(mu::Parser &parser)
{
	int *event, *event_orig;
	double *E;
	const char *varname;
	mu::varmap_type variables;
	mu::varmap_type::const_iterator iter;

	// find all event variables
	variables = parser.GetUsedVar();
	for(iter = variables.begin(); iter != variables.end(); ++iter) {
		varname = iter->first.c_str();
		if(strstr(varname, "event") == varname) {
			// create new event variable
			if(!(E = (double *) malloc(sizeof(double)))) {
				std::cerr << "error: unable to create stochastic events\n";
				exit(EXIT_FAILURE);
			}
			if(!(event = (int *) malloc(sizeof(int)))) {
				std::cerr << "error: unable to create stochastic events\n";
				exit(EXIT_FAILURE);
			}
			memcpy(&event_orig, &(iter->second), sizeof(int *));
			*event = *event_orig;
			memcpy(E, &event, sizeof(int *));
			// reassign event variable
			parser.RemoveVar(varname);
			parser.DefineVar(varname, E);
		}
	}
}

// free stochastic comparison event variables
void free_stochastic_comparisons_variables(mu::Parser &parser)
{
	int *event;
	double *E;
	const char *varname;
	mu::varmap_type variables;
	mu::varmap_type::const_iterator iter;

	// find all event variables
	variables = parser.GetUsedVar();
	for(iter = variables.begin(); iter != variables.end(); ++iter) {
		varname = iter->first.c_str();
		if(strstr(varname, "event") == varname) {
			// create new event variable
			E = iter->second;
			memcpy(&event, E, sizeof(int *));
			free(E);
			free(event);
		}
	}
}

//Auxiliary function to store the content of a file in a string
std::string ECSB::fileToString(std::string file)
{
    //Output string
    std::string stringFile;

    //String to store a line
    std::string line;

    //Reading the file line by line and storing it in stringFile
    std::ifstream inputFile(file.c_str());

    if (inputFile.is_open())
    {
        while (! inputFile.eof() )
        {
            getline (inputFile,line);
            stringFile += line;
        }
        inputFile.close();
    }
    else
    {
        std::cerr << "Unable to open file: " <<  file << std::endl;
        exit(EXIT_FAILURE);
    }

    return stringFile;
}

// Store the lines of a file in a map specifying line number and line
std::map<unsigned long, std::string> ECSB::fileToLines(std::string fileName)
{

    //Output map
    std::map<unsigned long, std::string> fileLines;

    //String to store a line and line number
    std::string line;
    unsigned long lineNumber = 1;

    //Reading the file line by line and storing it in the output map
    std::ifstream inputFile(fileName.c_str());

    if(inputFile.is_open())
    {
        while(!inputFile.eof())
        {
            getline(inputFile,line);
            fileLines[lineNumber] = line;
            ++lineNumber;
        }
        inputFile.close();
    }
    else
    {
        std::cerr << "Unable to open file: " << fileName << std::endl;
        exit(EXIT_FAILURE);
    }

    return fileLines;
    
}

// Find the line of a file where the string reporting an error starts
unsigned long ECSB::findLine(std::string errorStr, std::map<unsigned long,std::string> fileLines)
{
    std::string accum;

    for(std::map<unsigned long,std::string>::reverse_iterator line = fileLines.rbegin();
        line != fileLines.rend();
        ++line)
    {
        accum = line->second + accum;
        if(accum.find(errorStr) != accum.npos)
        {
            return line->first;
        }
    }

    std::cerr << "Unable to find error: " << errorStr << std::endl;
    exit(EXIT_FAILURE);

}



//Auxiliar substitution
std::map<std::string,std::string> ECSB::createSubstitution(std::vector<std::string> v1,
							 std::vector<std::string> v2)
{
  std::map<std::string,std::string> substitutionMap;

  if(v1.size() != v2.size())
    {
      std::cout << "Substitution vectors of different sizes for" << std::endl;
      std::cout << "v1 = ";
      for(std::vector<std::string>::iterator v = v1.begin(); v != v1.end(); ++v)
	std::cout << " " << *v;
      std::cout << std::endl <<  "v2 = ";
      for(std::vector<std::string>::iterator v = v2.begin(); v != v2.end(); ++v)
	std::cout << " " << *v;
      std::cout << std::endl;
      exit(1);
    }
  else
    {
      for(unsigned long i = 0; i != v1.size(); ++i)
	substitutionMap.insert(std::pair<std::string,std::string>(v1[i],v2[i]));
    }

  return substitutionMap;
}

//Auxiliar tokenizer
std::vector<std::string> ECSB::convertToVector(std::string str, std::string delimiters)
{
  std::vector<std::string> tokens;

  std::string::size_type pos_ini = str.find_first_not_of(delimiters,0);
  std::string::size_type pos_end = str.find_first_of(delimiters,pos_ini);

  while(pos_ini != str.npos || pos_end != str.npos)
    {
      tokens.push_back(str.substr(pos_ini,pos_end-pos_ini));
      pos_ini = str.find_first_not_of(delimiters,pos_end);
      pos_end = str.find_first_of(delimiters,pos_ini);
    }

  return tokens;
}


std::vector<std::string> ECSB::tokenizer(std::string output,
                                         std::string token)
{
    std::vector<std::string> outputVector;

    size_t pos1 = 0;
    size_t pos2 = output.find(token);

    while( pos2 != output.npos)
    {
        std::string result = output.substr(pos1,pos2-pos1);
        outputVector.push_back(result);

        pos1 = pos2 + 1;
        pos2 = output.find(token,pos1);
    }

    std::string result = output.substr(pos1,pos2-pos1);
    outputVector.push_back(result);

    return outputVector;
}

//Auxiliar vectors equality
int ECSB::equalityVector(std::vector<std::string> v1, std::vector<std::string> v2)
{
  if(v1.empty() && v2.empty())
    return 1;
  else if (v1.empty() && not(v2.empty()))
    return 0;
  else if (not(v1.empty()) && v2.empty())
    return 0;
  else
    {
      std::list<std::string> l1(v1.begin(),v1.end());
      l1.sort();
      std::list<std::string> l2(v2.begin(),v2.end());
      l2.sort();
      if(l1 == l2)
	return 1;
      else
	return 0;
    }
}

bool ECSB::isNumeric(std::string numericString)
{
     for(int i = 0; i < numericString.length(); ++i)
    {
        if(!(isdigit(numericString[i]) || numericString[i] == '.'))
        {
            return false; //Failure!!
        }
    }
   return true; //Success!!
}

std::string ECSB::removeSpaces(std::string str)
{
   size_t pos = str.find(' ');

   while(pos != str.npos)
   {
       str.erase(pos,1);
	    pos = str.find(' ');
   }

   pos = str.find("\t");

   while(pos != str.npos)
   {
       str.erase(pos,1);
	    pos = str.find("\t");
   }

   return str;
}
